
#include "em8300_reg.h"

typedef struct {
    void *ucode;
    int ucode_size;
} microcode_t;

typedef struct {
    int reg;
    int val;
} register_t;

#define EMCTL_IOCTL_INIT       _IOW('C',0,microcode_t)
#define EMCTL_IOCTL_READREG    _IORW('C',1,register_t)
#define EMCTL_IOCTL_WRITEREG   _IOW('C',2,register_t)
#define EMCTL_IOCTL_GETSTATUS  _IOR('C',3,char[1024])

#define EM8300_SUBDEVICE_CONTROL 0
#define EM8300_SUBDEVICE_VIDEO 1
#define EM8300_SUBDEVICE_AUDIO 2
#define EM8300_SUBDEVICE_SUBPICTURE 3

#ifndef PCI_VENDOR_ID_SIGMADESIGNS
#define PCI_VENDOR_ID_SIGMADESIGNS 0x1105
#define PCI_DEVICE_ID_SIGMADESIGNS_EM8300 0x8300
#endif

#define CLOCKGEN_SAMPFREQ_MASK 0xc0
#define CLOCKGEN_SAMPFREQ_66 0xc0
#define CLOCKGEN_SAMPFREQ_48 0x40
#define CLOCKGEN_SAMPFREQ_44 0x80
#define CLOCKGEN_SAMPFREQ_32 0x00
#define CLOCKGEN_ENABLE 0x10

#define MVCOMMAND_STOP 0x0
#define MVCOMMAND_11 0x11
#define MVCOMMAND_10 0x10
#define MVCOMMAND_START 3

#define MACOMMAND_STOP 0x0
#define MACOMMAND_START 0x2

#define IRQSTATUS_VIDEO 0x10
#define IRQSTATUS_AUDIO 0x8

#ifdef __KERNEL__

#define FIFOTYPE_AUDIO 1
#define FIFOTYPE_VIDEO 2

struct video_fifoslot_s {
    unsigned flags;
    unsigned physaddress_hi;
    unsigned physaddress_lo;
    unsigned slotsize;
};

struct audio_fifoslot_s {
    unsigned physaddress_hi;
    unsigned physaddress_lo;
    unsigned slotsize;
};

struct fifo_s {
    int valid;

    int type;
    int nslots;
    union {
	struct video_fifoslot_s *v;
	struct audio_fifoslot_s *a;
    } slots;
    int slotptrsize;
    int slotsize;

    int start;
    int *writeptr;
    int localwriteptr;
    int *readptr;
    int localreadptr;
    int threshold;

    char *fifobuffer;

#if LINUX_VERSION_CODE < 0x020314    
    struct wait_queue *wait;
#else
    wait_queue_head_t wait;
#endif	
    int waiting;
};


#define EM8300_MAX 4

#define EM8300_MAJOR 121
#define EM8300_LOGNAME "em8300"

#define DEBUG(x...) x

struct dicom_s {
    int luma;
    int chroma;
    int frametop;
    int framebottom;
    int frameleft;
    int frameright;
    int visibletop;
    int visiblebottom;
    int visibleleft;
    int visibleright;
    int tvout;
};


struct em8300_s
{
    char name[40];

    int inuse[4];
    int ucodeloaded;
    
    struct pci_dev *dev;
    ulong adr;
    volatile unsigned *mem;
    ulong memsize;

    /* Fifos */
    struct fifo_s mvfifo;
    struct fifo_s mafifo;
    struct fifo_s spfifo;
    
    struct dicom_s dicom;
    
    /* I2C bus 1*/
    struct i2c_algo_bit_data i2c_data_1;
    struct i2c_adapter i2c_ops_1;

    /* I2C bus 2*/
    struct i2c_algo_bit_data i2c_data_2;
    struct i2c_adapter i2c_ops_2;

    /* I2C clients */
    struct i2c_client *encoder;
    struct i2c_client *eeprom;

    /* Microcode registers */
    unsigned ucode_regs[MAX_UCODE_REGISTER];

    /* Interrupt */
    unsigned irqmask;

    /* Clockgenerator */
    int clockgen;
    
    /* Timing measurement */
    struct timeval tv,last_status_time;
    long irqtimediff;
    int irqcount;
    int frames;
    int scr;

    /* Audio */
    int swapbytes;
    
    int linecounter;
};


int em8300_i2c_init(struct em8300_s *em);
void em8300_i2c_exit(struct em8300_s *em);

/*
  Prototypes
*/

int em8300_waitfor(struct em8300_s *em, int reg, int val, int mask);

int em8300_audio_ioctl(struct em8300_s *em,unsigned int cmd, unsigned long arg);
int em8300_audio_open(struct em8300_s *em);
int em8300_audio_release(struct em8300_s *em);
void em8300_clockgen_write(struct em8300_s *em, int val);


#endif
